/*
 * JBoss Open Source Federation
 * The Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.demo.sessionsharing;

import java.util.Hashtable;
import java.util.Vector;

import javax.servlet.ServletContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

public class ShareGetDataAction extends Action {

	/**
	 * Get a cookie value, then get a shared data object from the context that created it.
	 * 
	 */
	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		HttpSession session = request.getSession();

		// get the shared the cookie value
		printCookies(request);		

		// get the shared session id
		String sessionid = getCookieValue(request, "JSESSIONID");
		System.out.println("sessionIDGetter: " + sessionid);
				
		request.setAttribute("sessionId", sessionid);

		if (request.getParameter("serialized") != null) {
			SerializedData data = (SerializedData) getDataFromContext(this
					.getServlet().getServletContext(), sessionid);

			request.setAttribute("objectValue", data);
		}

		return mapping.findForward("SUCCESS");
	}
	
	/**
	 * 
	 * utility method to get an object in the servlet context.
	 *  
	 * @param context
	 * @param sessionid
	 * 
	 */
	private Object getDataFromContext(ServletContext context,
			String sessionid) {

		// get the Context containing the shared session data
		ServletContext ssocontext = context.getContext("/wars-session-share");

		Object data = null;

		// read the shareddata
		Hashtable shareddata = (Hashtable) ssocontext
				.getAttribute("shared_data");
		if (shareddata != null) {
			// get the right object using the ssosessionid
			data = shareddata.get(sessionid);
		}

		return data;
	}
	
	/**
	 * 
	 * utility method to get a cookie value.
	 *  
	 * @param request
	 * @param name
	 * @return cookie value string
	 */
	private String getCookieValue(HttpServletRequest request, String name) {
		boolean found = false;
		String result = null;
		Cookie[] cookies = request.getCookies();
		if (cookies != null) {
			int i = 0;
			while (!found && i < cookies.length) {
				if (cookies[i].getName().equals(name)) {
					found = true;
					result = cookies[i].getValue();
					System.out.println("getCookieValue= " + result);
				}
				i++;
			}
		}
		return (result);
	}
	
	/**
	 * print the existing session cookies.
	 * 
	 * @param request
	 */
	private void printCookies(HttpServletRequest request)
	{
		Cookie[] cookies = request.getCookies();
		for (int i=0; i < cookies.length; i++)
		{
			System.out.println(cookies[i].getName());
		}
	}
}
